@@ -1,20 +1,10 @@
-def result(process, allow_error=False):
-    result = process.wait_for_result()
-    if allow_error or result.return_code == 0:
+def result(return_code, allow_error, output, stderr_output):
+    result = ExecutionResult(return_code, output, stderr_output)
+    if return_code == 0 or allow_error:
         return result
     else:
         raise result.to_error()
-
-
-class RunProcessError(RuntimeError):
-    def __init__(self, return_code, output, stderr_output):
-        message = "return code: {0}\noutput: {1}\nstderr output: {2}".format(
-            return_code, output, stderr_output)
-        super(type(self), self).__init__(message)
-        self.return_code = return_code
-        self.output = output
-        self.stderr_output = stderr_output
-
+        
 
 class ExecutionResult(object):
     def __init__(self, return_code, output, stderr_output):
@@ -28,3 +18,13 @@ class ExecutionResult(object):
             self.output, 
             self.stderr_output
         )
+        
+        
+class RunProcessError(RuntimeError):
+    def __init__(self, return_code, output, stderr_output):
+        message = "return code: {0}\noutput: {1}\nstderr output: {2}".format(
+            return_code, output, stderr_output)
+        super(type(self), self).__init__(message)
+        self.return_code = return_code
+        self.output = output
+        self.stderr_output = stderr_output
