@@ -32,6 +32,15 @@ class SshShell(object):
         self._private_key_file = private_key_file
         self._client = None
         self._connect_timeout = connect_timeout if not None else _ONE_MINUTE
+        self._closed = False
+
+    def __enter__(self):
+        return self
+        
+    def __exit__(self, *args):
+        self._closed = True
+        if self._client is not None:
+            self._client.close()
 
     def run(self, *args, **kwargs):
         return self.spawn(*args, **kwargs).wait_for_result()
@@ -40,15 +49,27 @@ class SshShell(object):
         stdout = kwargs.pop("stdout", None)
         stderr = kwargs.pop("stderr", None)
         allow_error = kwargs.pop("allow_error", False)
-        command_in_cwd = self._generate_run_command(*args, **kwargs)
+        store_pid = kwargs.pop("store_pid", False)
+        command_in_cwd = self._generate_run_command(*args, store_pid=store_pid, **kwargs)
         channel = self._get_ssh_transport().open_session()
         channel.exec_command(command_in_cwd)
-        return SshProcess(
+        
+        process_stdout = channel.makefile('rb')
+        if store_pid:
+            pid = int(process_stdout.readline().strip())
+            
+        process = SshProcess(
             channel,
             allow_error=allow_error,
+            process_stdout=process_stdout,
             stdout=stdout,
-            stderr=stderr
+            stderr=stderr,
+            shell=self,
         )
+        if store_pid:
+            process.pid = pid
+        
+        return process
     
     @contextlib.contextmanager
     def temporary_dir(self):
@@ -59,18 +80,30 @@ class SshShell(object):
         finally:
             self.run(["rm", "-rf", temp_dir])
     
-    def _generate_run_command(self, command_args, cwd="/", update_env={}, new_process_group=False):
-        command = " ".join(map(escape_sh, command_args))
+    def _generate_run_command(self, command_args, store_pid,
+            cwd=None, update_env={}, new_process_group=False):
+        commands = []
+
+        if store_pid:
+            commands.append("echo $$")
+
+        if cwd is not None:
+            commands.append("cd {0}".format(escape_sh(cwd)))
         
-        update_env_commands = " ".join([
-            "export {0}={1};".format(key, escape_sh(value))
+        update_env_commands = [
+            "export {0}={1}".format(key, escape_sh(value))
             for key, value in update_env.iteritems()
-        ])
+        ]
+        commands += update_env_commands
         
+        command = " ".join(map(escape_sh, command_args))
+        command = "exec {0}".format(command)
         if new_process_group:
             command = "setsid {0}".format(command)
+            
+        commands.append(command)
         
-        return "cd {0}; {1} {2}".format(cwd, update_env_commands, command)
+        return "; ".join(commands)
         
     
     def upload_dir(self, local_dir, remote_dir, ignore):
@@ -112,6 +145,8 @@ class SshShell(object):
     
     def _connect_ssh(self):
         if self._client is None:
+            if self._closed:
+                raise RuntimeError("Shell is closed")
             client = paramiko.SSHClient()
             client.load_system_host_keys()
             client.set_missing_host_key_policy(paramiko.WarningPolicy())
@@ -166,12 +201,13 @@ def escape_sh(value):
 
 
 class SshProcess(object):
-    def __init__(self, channel, allow_error, stdout, stderr):
+    def __init__(self, channel, allow_error, process_stdout, stdout, stderr, shell):
         self._channel = channel
         self._allow_error = allow_error
         self._stdin = channel.makefile('wb')
-        self._stdout = channel.makefile('rb')
+        self._stdout = process_stdout
         self._stderr = channel.makefile_stderr('rb')
+        self._shell = shell
         self._result = None
         
         self._io = IoHandler([
@@ -185,6 +221,9 @@ class SshProcess(object):
     def stdin_write(self, value):
         self._channel.sendall(value)
         
+    def send_signal(self, signal):
+        self._shell.run(["kill", "-{0}".format(signal), str(self.pid)])
+        
     def wait_for_result(self):
         if self._result is None:
             self._result = self._generate_result()
